==> suseRegister-1.3.tar.bz2: suseRegister-1.3/ZmdConfig.pm <==
package SUSE::Parser::ZmdConfig;
use strict;
use XML::Parser;
use SUSE::SRPrivate;

# The return value of parse is a hash reference on success
# and has the following format: 
#
# $VAR1 = {
#          'nVidia-Driver-SLE10-SP2' => {
#                                       'URL' => 'http://srv64.suse.de//repo/RPMMD/nVidia-Driver-SLE10-SP2',
#                                       'NAME' => 'nVidia-Driver-SLE10-SP2',
#                                       'TYPE' => 'zypp'
#                                     },
#          'local_nu_server' => {
#                               'URL' => 'http://srv64.suse.de/',
#                               'NAME' => 'local_nu_server',
#                               'CATALOGS' => {
#                                               'SLED10-SP2-Updates' => {
#                                                                       'URL' => 'http://srv64.suse.de//repo/$RCE/SLED10-SP2-Updates/sled-10-i586',
#                                                                       'NAME' => 'SLED10-SP2-Updates'
#                                                                     }
#                                             },
#                               'TYPE' => 'nu'
#                             },
#          'ATI-Driver-SLE10-SP2' => {
#                                    'URL' => 'http://srv64.suse.de//repo/RPMMD/ATI-Driver-SLE10-SP2',
#                                    'NAME' => 'ATI-Driver-SLE10-SP2',
#                                    'TYPE' => 'zypp'
#                                  }
#         };


# constructor
sub new
{
    my $pkgname = shift;
    my %opt   = @_;
    my $self  = {};

    $self->{CURRENT}   = undef;
    $self->{FINAL}     = undef;
    $self->{ELEMENT}   = undef;
    $self->{CATURL}    = undef;
    $self->{CATNAME}   = undef;
    $self->{CTX}       = undef;
    
    if(exists $opt{ctx} && defined $opt{ctx} && $opt{ctx})
    {
        $self->{CTX} = $opt{ctx};
    }
    bless($self);
    return $self;
}

# parses a xml resource
sub parse()
{
    my $self     = shift;
    my $xml      = shift;

    if (!defined $xml)
    {
        SUSE::SRPrivate::printLog($self->{CTX}, "error", "Invalid content");
        return 1;
    }

    my $parser = XML::Parser->new( Handlers =>
                                   {
                                    Start=> sub { handle_start_tag($self, @_) },
                                    Char => sub { handle_char_tag($self, @_) },
                                    End=> sub { handle_end_tag($self, @_) },
                                   });

    eval
    {
        $parser->parse( $xml );
    };
    if ($@) {
        # ignore the errors, but print them
        chomp($@);
        SUSE::SRPrivate::printLog($self->{CTX}, "warn", "SUSE::Parser::RepoList Invalid XML \n$xml\n: $@");
    }
    return $self->{FINAL};
}

# handles XML reader start tag events
sub handle_start_tag()
{
    my $self = shift;
    my( $expat, $element, %attrs ) = @_;

    if(lc($element) eq "service")
    {
        $self->{ELEMENT} = "service";
        
        if(exists $attrs{id} && defined $attrs{id})
        {
            $self->{CURRENT}->{NAME} = $attrs{id};
            $self->{CURRENT}->{ALIAS} = $attrs{id};
        }
        if(exists $attrs{type} && defined $attrs{type})
        {
            $self->{CURRENT}->{TYPE} = $attrs{type};
        }
        $self->{CURRENT}->{URL} = "";

    }    
    elsif(lc($element) eq "param")
    {
        if(exists $attrs{id} && defined $attrs{id})
        {
            $self->{ELEMENT} = $attrs{id};
        }
        elsif(exists $attrs{name} && defined $attrs{name})
        {
            if(exists $attrs{url} && defined $attrs{url})
            {
                $self->{CATURL} = $attrs{url};
            }
            $self->{ELEMENT} = $attrs{name};
        }
    }
}

sub handle_char_tag
{
    my $self = shift;
    my( $expat, $string) = @_;

    chomp($string);
    return if($string =~ /^\s*$/);
    $string =~ s/^\s*//;
    $string =~ s/\s*$//;
    
    return if(! defined $self->{ELEMENT});

    if($self->{ELEMENT} eq "url")
    {
        $self->{CURRENT}->{URL} .= $string;
    }
    elsif($self->{ELEMENT} eq "catalog")
    {
        $self->{CATNAME} .= $string;
    }
}

sub handle_end_tag
{
    my( $self, $expat, $element ) = @_;

    if(lc($element) eq "service")
    {
        # move CURRENT to FINAL

        if($self->{CURRENT}->{TYPE} eq "zypp" || $self->{CURRENT}->{TYPE} eq "yum")
        {
            my $name = "";
            if(exists $self->{CURRENT}->{CATALOGS} && ref($self->{CURRENT}->{CATALOGS}) eq "HASH")
            {
                $name = (keys %{$self->{CURRENT}->{CATALOGS}})[0];
            }
            else
            {
                $name = $self->{CURRENT}->{NAME};
            }
            $self->{FINAL}->{$name}->{NAME}  = $name;
            $self->{FINAL}->{$name}->{ALIAS} = $name;
            $self->{FINAL}->{$name}->{TYPE}  = $self->{CURRENT}->{TYPE};
            $self->{FINAL}->{$name}->{URL}   = $self->{CURRENT}->{URL};
        }
        elsif($self->{CURRENT}->{TYPE} eq "nu")
        {
            my $alias = $self->{CURRENT}->{ALIAS};
            $self->{FINAL}->{$alias}->{NAME}  = $self->{CURRENT}->{NAME};
            $self->{FINAL}->{$alias}->{ALIAS} = $self->{CURRENT}->{ALIAS};
            $self->{FINAL}->{$alias}->{TYPE}  = $self->{CURRENT}->{TYPE};
            $self->{FINAL}->{$alias}->{URL}   = $self->{CURRENT}->{URL};
            
            if(exists $self->{CURRENT}->{CATALOGS} && ref($self->{CURRENT}->{CATALOGS}) eq "HASH")
            {
                foreach my $catname (keys %{$self->{CURRENT}->{CATALOGS}})
                {
                    $self->{FINAL}->{$alias}->{CATALOGS}->{$catname}->{URL}   = $self->{CURRENT}->{CATALOGS}->{$catname}->{URL};
                    $self->{FINAL}->{$alias}->{CATALOGS}->{$catname}->{NAME}  = $self->{CURRENT}->{CATALOGS}->{$catname}->{NAME};
                    $self->{FINAL}->{$alias}->{CATALOGS}->{$catname}->{ALIAS} = $self->{CURRENT}->{CATALOGS}->{$catname}->{ALIAS};
                }
            }
        }
        
        $self->{ELEMENT}   = undef;
        $self->{CURRENT}   = undef;
        $self->{SERVICEID} = undef;
    }
    elsif(lc($element) eq "param")
    {
        if($self->{ELEMENT} eq "url")
        {
            $self->{ELEMENT} = undef;
        }
        elsif($self->{ELEMENT} eq "catalog")
        {
            my $catalias = "";
            if($self->{CURRENT}->{TYPE} eq "nu")
            {
                $catalias = $self->{CURRENT}->{ALIAS}.":".$self->{CATNAME};
            }
            else
            {
                $catalias = $self->{CATNAME};
            }            
            
            $self->{CURRENT}->{CATALOGS}->{$catalias}->{NAME}  = $self->{CATNAME};
            $self->{CURRENT}->{CATALOGS}->{$catalias}->{ALIAS} = $catalias;
            $self->{CURRENT}->{CATALOGS}->{$catalias}->{URL}   = $self->{CATURL};


            $self->{ELEMENT} = undef;
            $self->{CATNAME} = "";
            $self->{CATURL}  = "";
        }
    }
}

1;

==> suseRegister-1.4.tar.bz2: suseRegister-1.4/ZmdConfig.pm <==
package SUSE::Parser::ZmdConfig;
use strict;
use XML::Parser;
use SUSE::SRPrivate;

# The return value of parse is a hash reference on success
# and has the following format: 
#
# $VAR1 = {
#          'nVidia-Driver-SLE10-SP2' => {
#                                       'URL' => 'http://srv64.suse.de//repo/RPMMD/nVidia-Driver-SLE10-SP2',
#                                       'NAME' => 'nVidia-Driver-SLE10-SP2',
#                                       'TYPE' => 'zypp'
#                                     },
#          'local_nu_server' => {
#                               'URL' => 'http://srv64.suse.de/',
#                               'NAME' => 'local_nu_server',
#                               'CATALOGS' => {
#                                               'SLED10-SP2-Updates' => {
#                                                                       'URL' => 'http://srv64.suse.de//repo/$RCE/SLED10-SP2-Updates/sled-10-i586',
#                                                                       'NAME' => 'SLED10-SP2-Updates'
#                                                                     }
#                                             },
#                               'TYPE' => 'nu'
#                             },
#          'ATI-Driver-SLE10-SP2' => {
#                                    'URL' => 'http://srv64.suse.de//repo/RPMMD/ATI-Driver-SLE10-SP2',
#                                    'NAME' => 'ATI-Driver-SLE10-SP2',
#                                    'TYPE' => 'zypp'
#                                  }
#         };


# constructor
sub new
{
    my $pkgname = shift;
    my %opt   = @_;
    my $self  = {};

    $self->{CURRENT}   = undef;
    $self->{FINAL}     = undef;
    $self->{ELEMENT}   = undef;
    $self->{CATURL}    = undef;
    $self->{CATNAME}   = undef;
    $self->{CTX}       = undef;
    $self->{STATUSXML} = "";
    $self->{CREDXML}   = "";
    $self->{STATUS}    = undef;
    $self->{PRODUCTSTATUS} = undef;
    $self->{ISSTATUS}  = 0;
    $self->{ISCRED}    = 0;

    if(exists $opt{ctx} && defined $opt{ctx} && $opt{ctx})
    {
        $self->{CTX} = $opt{ctx};
    }
    bless($self);
    return $self;
}

sub statusxml
{
  my $self     = shift;
  # if STATUS is defined, the server send a status return status or ""
  # if no status was send return undef
  return ((defined $self->{STATUS})?$self->{STATUSXML}:undef);
}

sub credentialsxml
{
  my $self     = shift;
  return $self->{CREDXML};
}

sub status
{
  my $self     = shift;
  return $self->{STATUS};
}

# parses a xml resource
sub parse()
{
    my $self     = shift;
    my $xml      = shift;

    if (!defined $xml)
    {
        SUSE::SRPrivate::printLog($self->{CTX}, "error", "Invalid content");
        return 1;
    }

    my $parser = XML::Parser->new( Handlers =>
                                   {
                                    Start=> sub { handle_start_tag($self, @_) },
                                    Char => sub { handle_char_tag($self, @_) },
                                    End=> sub { handle_end_tag($self, @_) },
                                   });

    eval
    {
        $parser->parse( $xml );
    };
    if ($@) {
        # ignore the errors, but print them
        chomp($@);
        SUSE::SRPrivate::printLog($self->{CTX}, "warn", "SUSE::Parser::RepoList Invalid XML \n$xml\n: $@");
    }
    return $self->{FINAL};
}

# handles XML reader start tag events
sub handle_start_tag()
{
    my $self = shift;
    my( $expat, $element, %attrs ) = @_;

    if(lc($element) eq "service")
    {
        $self->{ELEMENT} = "service";

        if(exists $attrs{id} && defined $attrs{id})
        {
            $self->{CURRENT}->{NAME} = $attrs{id};
            $self->{CURRENT}->{ALIAS} = $attrs{id};
        }
        if(exists $attrs{type} && defined $attrs{type})
        {
            $self->{CURRENT}->{TYPE} = $attrs{type};
        }
        $self->{CURRENT}->{URL} = "";

    }    
    elsif(lc($element) eq "param")
    {
        if(exists $attrs{id} && defined $attrs{id})
        {
            $self->{ELEMENT} = $attrs{id};
        }
        elsif(exists $attrs{name} && defined $attrs{name})
        {
            if(exists $attrs{url} && defined $attrs{url})
            {
                $self->{CATURL} = $attrs{url};
            }
            $self->{ELEMENT} = $attrs{name};
        }
    }
    elsif(lc($element) eq "status")
    {
      $self->{ISSTATUS} = 1;
      $self->{STATUSXML} = '<?xml version="1.0" encoding="UTF-8"?>'."\n";
      $self->{STATUS} = {};
    }
    elsif(lc($element) eq "credentials")
    {
      $self->{ISCRED} = 1;
      $self->{CREDXML} = '<?xml version="1.0" encoding="UTF-8"?>'."\n";
    }
    elsif($self->{ISSTATUS} && lc($element) eq "productstatus")
    {
      #$self->{ELEMENT} = "productstatus";
      $self->{PRODUCTSTATUS} = "";
      if(exists $attrs{"product"} && defined $attrs{"product"})
      {
        $self->{PRODUCTSTATUS} .= $attrs{"product"};
      }
      if(exists $attrs{"version"} && defined $attrs{"version"})
      {
        $self->{PRODUCTSTATUS} .= "-".$attrs{"version"};
      }
      if(exists $attrs{"release"} && defined $attrs{"release"})
      {
        $self->{PRODUCTSTATUS} .= "-".$attrs{"release"};
      }
      if(exists $attrs{"arch"} && defined $attrs{"arch"})
      {
        $self->{PRODUCTSTATUS} .= "-".$attrs{"arch"};
      }
      
      $self->{STATUS}->{$self->{PRODUCTSTATUS}} = {};
      
      if(exists $attrs{"result"} && defined $attrs{"result"})
      {
        $self->{STATUS}->{$self->{PRODUCTSTATUS}}->{result} = $attrs{"result"};
      }
      if(exists $attrs{"errorcode"} && defined $attrs{"errorcode"})
      {
        $self->{STATUS}->{$self->{PRODUCTSTATUS}}->{errorcode} = $attrs{"errorcode"};
      }
      else
      {
        $self->{STATUS}->{$self->{PRODUCTSTATUS}}->{errorcode} = "UNKNOWN";
      }
    }
    elsif($self->{ISSTATUS} && lc($element) eq "message" && defined $self->{PRODUCTSTATUS})
    {
      $self->{ELEMENT} = "message";
      $self->{STATUS}->{$self->{PRODUCTSTATUS}}->{message} = "";
    }

    if( $self->{ISSTATUS} )
    {
      $self->{STATUSXML} .= $expat->original_string;
    }
    elsif( $self->{ISCRED} )
    {
      $self->{CREDXML} .= $expat->original_string;
    }
}

sub handle_char_tag
{
    my $self = shift;
    my( $expat, $string) = @_;

    if( $self->{ISSTATUS} )
    {
      $self->{STATUSXML} .= $expat->original_string;
    }
    elsif( $self->{ISCRED} )
    {
      $self->{CREDXML} .= $expat->original_string;
    }

    chomp($string);
    return if($string =~ /^\s*$/);
    $string =~ s/^\s*//;
    $string =~ s/\s*$//;

    return if(! defined $self->{ELEMENT});

    if($self->{ELEMENT} eq "url")
    {
        $self->{CURRENT}->{URL} .= $string;
    }
    elsif($self->{ELEMENT} eq "catalog")
    {
        $self->{CATNAME} .= $string;
    }
    elsif($self->{ISSTATUS} && $self->{ELEMENT} eq "message")
    {
      $self->{STATUS}->{$self->{PRODUCTSTATUS}}->{message} .= $string ;
    }
}

sub handle_end_tag
{
    my( $self, $expat, $element ) = @_;

    if( $self->{ISSTATUS} )
    {
      $self->{STATUSXML} .= $expat->original_string;
    }
    elsif( $self->{ISCRED} )
    {
      $self->{CREDXML} .= $expat->original_string;
    }

    if(lc($element) eq "service")
    {
        # move CURRENT to FINAL

        if($self->{CURRENT}->{TYPE} eq "zypp")
        {
            my $name = "";
            if(exists $self->{CURRENT}->{CATALOGS} && ref($self->{CURRENT}->{CATALOGS}) eq "HASH")
            {
                $name = (keys %{$self->{CURRENT}->{CATALOGS}})[0];
            }
            else
            {
                $name = $self->{CURRENT}->{NAME};
            }
            $self->{FINAL}->{$name}->{NAME}  = $name;
            $self->{FINAL}->{$name}->{ALIAS} = $name;
            $self->{FINAL}->{$name}->{TYPE}  = $self->{CURRENT}->{TYPE};
            $self->{FINAL}->{$name}->{URL}   = $self->{CURRENT}->{URL};
        }
        elsif($self->{CURRENT}->{TYPE} eq "nu")
        {
            my $alias = $self->{CURRENT}->{ALIAS};
            $self->{FINAL}->{$alias}->{NAME}  = $self->{CURRENT}->{NAME};
            $self->{FINAL}->{$alias}->{ALIAS} = $self->{CURRENT}->{ALIAS};
            $self->{FINAL}->{$alias}->{TYPE}  = $self->{CURRENT}->{TYPE};
            $self->{FINAL}->{$alias}->{URL}   = $self->{CURRENT}->{URL};
            
            if(exists $self->{CURRENT}->{CATALOGS} && ref($self->{CURRENT}->{CATALOGS}) eq "HASH")
            {
                foreach my $catname (keys %{$self->{CURRENT}->{CATALOGS}})
                {
                    $self->{FINAL}->{$alias}->{CATALOGS}->{$catname}->{URL}   = $self->{CURRENT}->{CATALOGS}->{$catname}->{URL};
                    $self->{FINAL}->{$alias}->{CATALOGS}->{$catname}->{NAME}  = $self->{CURRENT}->{CATALOGS}->{$catname}->{NAME};
                    $self->{FINAL}->{$alias}->{CATALOGS}->{$catname}->{ALIAS} = $self->{CURRENT}->{CATALOGS}->{$catname}->{ALIAS};
                }
            }
        }
        
        $self->{ELEMENT}   = undef;
        $self->{CURRENT}   = undef;
        $self->{SERVICEID} = undef;
    }
    elsif(lc($element) eq "param")
    {
        if($self->{ELEMENT} eq "url")
        {
            $self->{ELEMENT} = undef;
        }
        elsif($self->{ELEMENT} eq "catalog")
        {
            my $catalias = "";
            if($self->{CURRENT}->{TYPE} eq "nu")
            {
                $catalias = $self->{CURRENT}->{ALIAS}.":".$self->{CATNAME};
            }
            else
            {
                $catalias = $self->{CATNAME};
            }
            
            $self->{CURRENT}->{CATALOGS}->{$catalias}->{NAME}  = $self->{CATNAME};
            $self->{CURRENT}->{CATALOGS}->{$catalias}->{ALIAS} = $catalias;
            $self->{CURRENT}->{CATALOGS}->{$catalias}->{URL}   = $self->{CATURL};


            $self->{ELEMENT} = undef;
            $self->{CATNAME} = "";
            $self->{CATURL}  = "";
        }
    }
    elsif($self->{ISSTATUS} && lc($element) eq "status")
    {
      $self->{ISSTATUS} = 0;
    }
    elsif($self->{ISSTATUS} && lc($element) eq "productstatus")
    {
      $self->{PRODUCTSTATUS} = undef;
    }
    elsif($self->{ISSTATUS} && lc($element) eq "message")
    {
      $self->{ELEMENT} = undef;
    }
    elsif($self->{ISCRED} && lc($element) eq "credentials")
    {
      $self->{ISCRED} = 0;
    }
}

1;
